@@ -26,6 +26,11 @@ class Subtitle(object):
     SUBRIP_EXTENTIONS = [".srt"]
     TTML_EXTENSIONS = [".xml", ".ttml", ".dfxp"]
     WEBVTT_EXTENSIONS = [".vtt"]
+    SSA_EXTENTIONS = [".ssa"]
+    ADVANCED_SSA_EXTENTIONS = [".ass"]
+    MICRODVD_EXTENSIONS = [".sub"]
+    MPL2_EXTENSIONS = [".txt"]
+    TMP_EXTENSIONS = [".tmp"]
 
     def __init__(self, secret, subtitle_file_path, subtitle_format):
         """Subtitle object initialiser.
@@ -51,6 +56,16 @@ class Subtitle(object):
             self.__subs = self.__convert_ttml_to_subs(subtitle_file_path)
         elif subtitle_format == "webvtt":
             self.__subs = self.__convert_vtt_to_subs(subtitle_file_path)
+        elif subtitle_format == "ssa":
+            self.__subs = self.__convert_ssa_to_subs(subtitle_file_path)
+        elif subtitle_format == "ass":
+            self.__subs = self.__convert_ass_to_subs(subtitle_file_path)
+        elif subtitle_format == "microdvd":
+            self.__subs = self.__convert_microdvd_to_subs(subtitle_file_path)
+        elif subtitle_format == "mpl2":
+            self.__subs = self.__convert_mpl2_to_subs(subtitle_file_path)
+        elif subtitle_format == "tmp":
+            self.__subs = self.__convert_tmp_to_subs(subtitle_file_path)
         else:
             raise UnsupportedFormatException(
                 "Unknown subtitle format for file: {}".format(subtitle_file_path)
@@ -106,6 +121,71 @@ class Subtitle(object):
 
         return cls(cls.__secret, subtitle_file_path, "webvtt")
 
+    @classmethod
+    def load_ssa(cls, subtitle_file_path):
+        """Load a SubStation Alpha v4.0 subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "ssa")
+
+    @classmethod
+    def load_ass(cls, subtitle_file_path):
+        """Load a Advanced SubStation Alpha v4.0+ subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "ass")
+
+    @classmethod
+    def load_microdvd(cls, subtitle_file_path):
+        """Load a MicroDVD subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "microdvd")
+
+    @classmethod
+    def load_mpl2(cls, subtitle_file_path):
+        """Load a MPL2 subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "mpl2")
+
+    @classmethod
+    def load_tmp(cls, subtitle_file_path):
+        """Load a TMP subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "tmp")
+
     @classmethod
     def load(cls, subtitle_file_path):
         """Load a SubRip or TTML subtitle file based on the file extension.
@@ -124,6 +204,16 @@ class Subtitle(object):
             return cls(cls.__secret, subtitle_file_path, "ttml")
         elif file_extension in cls.WEBVTT_EXTENSIONS:
             return cls(cls.__secret, subtitle_file_path, "webvtt")
+        elif file_extension in cls.SSA_EXTENTIONS:
+            return cls(cls.__secret, subtitle_file_path, "ssa")
+        elif file_extension in cls.ADVANCED_SSA_EXTENTIONS:
+            return cls(cls.__secret, subtitle_file_path, "ass")
+        elif file_extension in cls.MICRODVD_EXTENSIONS:
+            return cls(cls.__secret, subtitle_file_path, "microdvd")
+        elif file_extension in cls.MPL2_EXTENSIONS:
+            return cls(cls.__secret, subtitle_file_path, "mpl2")
+        elif file_extension in cls.TMP_EXTENSIONS:
+            return cls(cls.__secret, subtitle_file_path, "tmp")
         else:
             return cls(cls.__secret, subtitle_file_path, "unknown")
 
@@ -152,11 +242,7 @@ class Subtitle(object):
         if file_extension.lower() in cls.SUBRIP_EXTENTIONS:
             subs = cls(cls.__secret, subtitle_file_path, "subrip").subs
             subs.shift(seconds=seconds)
-            if shifted_subtitle_file_path is None:
-                shifted_subtitle_file_path = subtitle_file_path.replace(
-                    file_extension, "{}{}".format(suffix, file_extension)
-                )
-            Subtitle.export_subtitle(subtitle_file_path, subs, shifted_subtitle_file_path)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
             return shifted_subtitle_file_path
         elif file_extension.lower() in cls.TTML_EXTENSIONS:
             subs = cls(cls.__secret, subtitle_file_path, "ttml").subs
@@ -180,11 +266,32 @@ class Subtitle(object):
         elif file_extension.lower() in cls.WEBVTT_EXTENSIONS:
             subs = cls(cls.__secret, subtitle_file_path, "webvtt").subs
             subs.shift(seconds=seconds)
-            if shifted_subtitle_file_path is None:
-                shifted_subtitle_file_path = subtitle_file_path.replace(
-                    file_extension, "{}{}".format(suffix, file_extension)
-                )
-            Subtitle.export_subtitle(subtitle_file_path, subs, shifted_subtitle_file_path)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
+            return shifted_subtitle_file_path
+        elif file_extension.lower() in cls.SSA_EXTENTIONS:
+            subs = cls(cls.__secret, subtitle_file_path, "ssa").subs
+            subs.shift(seconds=seconds)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
+            return shifted_subtitle_file_path
+        elif file_extension.lower() in cls.ADVANCED_SSA_EXTENTIONS:
+            subs = cls(cls.__secret, subtitle_file_path, "ass").subs
+            subs.shift(seconds=seconds)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
+            return shifted_subtitle_file_path
+        elif file_extension.lower() in cls.MICRODVD_EXTENSIONS:
+            subs = cls(cls.__secret, subtitle_file_path, "microdvd").subs
+            subs.shift(seconds=seconds)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
+            return shifted_subtitle_file_path
+        elif file_extension.lower() in cls.MPL2_EXTENSIONS:
+            subs = cls(cls.__secret, subtitle_file_path, "mpl2").subs
+            subs.shift(seconds=seconds)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
+            return shifted_subtitle_file_path
+        elif file_extension.lower() in cls.TMP_EXTENSIONS:
+            subs = cls(cls.__secret, subtitle_file_path, "tmp").subs
+            subs.shift(seconds=seconds)
+            Subtitle.__export_with_format(subs, subtitle_file_path, shifted_subtitle_file_path, file_extension, suffix)
             return shifted_subtitle_file_path
         else:
             raise UnsupportedFormatException(
@@ -192,13 +299,14 @@ class Subtitle(object):
             )
 
     @staticmethod
-    def export_subtitle(source_file_path, subs, target_file_path):
+    def export_subtitle(source_file_path, subs, target_file_path, frame_rate=25.0):
         """Export subtitle in the format determined by the file extension.
 
         Arguments:
             source_file_path {string} -- The path to the original subtitle file.
             subs {list} -- A list of SubRipItems.
             target_file_path {string} -- The path to the exported subtitle file.
+            frame_rate {float} -- The frame rate for frame-based subtitle formats {default: 25.0}.
         """
 
         filename, file_extension = os.path.splitext(source_file_path.lower())
@@ -234,6 +342,41 @@ class Subtitle(object):
                 Utils.srt2vtt(path, target_file_path)
             finally:
                 os.remove(path)
+        elif file_extension in Subtitle.SSA_EXTENTIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding="utf8")
+                Utils.srt2ssa(path, target_file_path)
+            finally:
+                os.remove(path)
+        elif file_extension in Subtitle.ADVANCED_SSA_EXTENTIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding="utf8")
+                Utils.srt2ass(path, target_file_path)
+            finally:
+                os.remove(path)
+        elif file_extension in Subtitle.MICRODVD_EXTENSIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding="utf8")
+                Utils.srt2microdvd(path, target_file_path, frame_rate=frame_rate)
+            finally:
+                os.remove(path)
+        elif file_extension in Subtitle.MPL2_EXTENSIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding="utf8")
+                Utils.srt2mpl2(path, target_file_path)
+            finally:
+                os.remove(path)
+        elif file_extension in Subtitle.TMP_EXTENSIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding="utf8")
+                Utils.srt2tmp(path, target_file_path)
+            finally:
+                os.remove(path)
         else:
             raise UnsupportedFormatException(
                 "Unknown subtitle format for file: {}".format(source_file_path)
@@ -319,10 +462,7 @@ class Subtitle(object):
                 Returns:
                     {list} -- A list of SubRipItems.
                 """
-        try:
-            return pysrt.open(subrip_file_path, encoding="utf-8")
-        except Exception as e:
-            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % subrip_file_path) from e
+        return Subtitle.__get_srt_subs(subrip_file_path)
 
     @staticmethod
     def __convert_ttml_to_subs(ttml_file_path):
@@ -338,13 +478,7 @@ class Subtitle(object):
         _, path = tempfile.mkstemp()
         Utils.ttml2srt(ttml_file_path, path)
 
-        try:
-            subs = pysrt.open(path, encoding="utf-8")
-        except Exception as e:
-            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % ttml_file_path) from e
-        finally:
-            os.remove(path)
-        return subs
+        return Subtitle.__get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_vtt_to_subs(vtt_file_path):
@@ -360,10 +494,108 @@ class Subtitle(object):
         _, path = tempfile.mkstemp()
         Utils.vtt2srt(vtt_file_path, path)
 
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_ssa_to_subs(ssa_file_path):
+        """Convert a subtitle file from the SubStation Alpha v4.0 format to the SubRip format
+
+        Arguments:
+            ass_file_path {string} -- The path to the SubStation Alpha v4.0 subtitle file.
+
+        Returns:
+            {list} -- A list of SubRipItems.
+        """
+
+        _, path = tempfile.mkstemp()
+        path = "%s.srt" % path
+        Utils.ssa2srt(ssa_file_path, path)
+
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_ass_to_subs(ass_file_path):
+        """Convert a subtitle file from the Advanced SubStation Alpha v4.0+ format to the SubRip format
+
+        Arguments:
+            ass_file_path {string} -- The path to the Advanced SubStation Alpha v4.0+ subtitle file.
+
+        Returns:
+            {list} -- A list of SubRipItems.
+        """
+
+        _, path = tempfile.mkstemp()
+        path = "%s.srt" % path
+        Utils.ass2srt(ass_file_path, path)
+
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_microdvd_to_subs(microdvd_file_path):
+        """Convert a subtitle file from the MicroDVD format to the SubRip format
+
+        Arguments:
+            microdvd_file_path {string} -- The path to the MicroDVD subtitle file.
+
+        Returns:
+            {list} -- A list of SubRipItems.
+        """
+
+        _, path = tempfile.mkstemp()
+        path = "%s.srt" % path
+        Utils.microdvd2srt(microdvd_file_path, path)
+
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_mpl2_to_subs(mpl2_file_path):
+        """Convert a subtitle file from the MPL2 format to the SubRip format
+
+        Arguments:
+            mpl2_file_path {string} -- The path to the MPL2 subtitle file.
+
+        Returns:
+            {list} -- A list of SubRipItems.
+        """
+
+        _, path = tempfile.mkstemp()
+        path = "%s.srt" % path
+        Utils.mpl22srt(mpl2_file_path, path)
+
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_tmp_to_subs(tmp_file_path):
+        """Convert a subtitle file from the TMP format to the SubRip format
+
+        Arguments:
+            tmp_file_path {string} -- The path to the TMP subtitle file.
+
+        Returns:
+            {list} -- A list of SubRipItems.
+        """
+
+        _, path = tempfile.mkstemp()
+        path = "%s.srt" % path
+        Utils.tmp2srt(tmp_file_path, path)
+
+        return Subtitle.__get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __export_with_format(subs, source_file_path, target_file_path, file_extension, suffix):
+        if target_file_path is None:
+            target_file_path = source_file_path.replace(
+                file_extension, "{}{}".format(suffix, file_extension)
+            )
+        Subtitle.export_subtitle(source_file_path, subs, target_file_path)
+
+    @staticmethod
+    def __get_srt_subs(subrip_file_path, housekeep=False):
         try:
-            subs = pysrt.open(path, encoding="utf-8")
+            subs = pysrt.open(subrip_file_path, encoding="utf-8")
         except Exception as e:
-            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % vtt_file_path) from e
+            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % subrip_file_path) from e
         finally:
-            os.remove(path)
+            if housekeep:
+                os.remove(subrip_file_path)
         return subs
